/* ***********************************************************
 * This file was automatically generated on 2012-10-12.      *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generator git on tinkerforge.com                   *
 *************************************************************/

#ifndef BRICKLET_DUAL_RELAY_H
#define BRICKLET_DUAL_RELAY_H

#include "ip_connection.h"

/**
 * \defgroup BrickletDualRelay DualRelay Bricklet
 */

/**
 * \ingroup BrickletDualRelay
 *
 * Device for controlling two relays
 */
typedef Device DualRelay;

/**
 * \ingroup BrickletDualRelay
 *
 * This callback is triggered whenever a monoflop timer reaches 0. The 
 * parameter contain the relay (1 or 2) and the current state of the relay 
 * (the state after the monoflop).
 * 
 * .. versionadded:: 1.1.1~(Plugin)
 */
#define DUAL_RELAY_CALLBACK_MONOFLOP_DONE 5

/**
 * \ingroup BrickletDualRelay
 *
 * Creates an object with the unique device ID \c uid. This object can then be
 * added to the IP connection.
 */
void dual_relay_create(DualRelay *dual_relay, const char *uid);

/**
 * \ingroup BrickletDualRelay
 *
 * Sets the state of the relays, *true* means on and *false* means off. 
 * For example: (true, false) turns relay 1 on and relay 2 off.
 * 
 * If you just want to set one of the relays and don't know the current state
 * of the other relay, you can get the state with {@link dual_relay_get_state}.
 * 
 * Running monoflop timers will be overwritten if this function is called.
 * 
 * The default value is (false, false).
 */
int dual_relay_set_state(DualRelay *dual_relay, bool relay1, bool relay2);

/**
 * \ingroup BrickletDualRelay
 *
 * Returns the state of the relays, *true* means on and *false* means off.
 */
int dual_relay_get_state(DualRelay *dual_relay, bool *ret_relay1, bool *ret_relay2);

/**
 * \ingroup BrickletDualRelay
 *
 * The first parameter can be 1 or 2 (relay 1 or relay 2). The second parameter 
 * is the desired state of the relay (*true* means on and *false* means off).
 * The third parameter indicates the time (in ms) that the relay should hold 
 * the state.
 * 
 * If this function is called with the parameters (1, true, 1500):
 * Relay 1 will turn on and in 1.5s it will turn off again.
 * 
 * A monoflop can be used as a failsafe mechanism. For example: Lets assume you 
 * have a RS485 bus and a Dual Relay Bricklet connected to one of the slave 
 * stacks. You can now call this function every second, with a time parameter
 * of two seconds. The relay will be on all the time. If now the RS485 
 * connection is lost, the relay will turn off in at most two seconds.
 * 
 * .. versionadded:: 1.1.1~(Plugin)
 */
int dual_relay_set_monoflop(DualRelay *dual_relay, uint8_t relay, bool state, uint32_t time);

/**
 * \ingroup BrickletDualRelay
 *
 * Returns (for the given relay) the current state and the time as set by 
 * {@link dual_relay_set_monoflop} as well as the remaining time until the state flips.
 * 
 * If the timer is not running currently, the remaining time will be returned
 * as 0.
 * 
 * .. versionadded:: 1.1.1~(Plugin)
 */
int dual_relay_get_monoflop(DualRelay *dual_relay, uint8_t relay, bool *ret_state, uint32_t *ret_time, uint32_t *ret_time_remaining);

/**
 * \ingroup BrickletDualRelay
 *
 * Returns the name (including the hardware version), the firmware version
 * and the binding version of the device. The firmware and binding versions are
 * given in arrays of size 3 with the syntax [major, minor, revision].
 */
int dual_relay_get_version(DualRelay *dual_relay, char ret_name[40], uint8_t ret_firmware_version[3], uint8_t ret_binding_version[3]);

/**
 * \ingroup BrickletDualRelay
 *
 * Registers a callback with ID \c id to the function \c callback.
 */
void dual_relay_register_callback(DualRelay *dual_relay, uint8_t id, void *callback);

#endif
